/*****************************************************************************************************************************
--
--    File Name    : axi_buffer.v

--    Description  : This module is the AXI buffer
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module axi_buffer #(parameter g_AXI_BUFF_AWIDTH = 12,
                    parameter AXI_BUFF_DEPTH = 960,
                    parameter g_AXI_DWIDTH = 64)
                  (input wclock,
                   input rclock,
                   input we,
                   input[g_AXI_BUFF_AWIDTH - 1 : 0] rd_addr,
                   input[g_AXI_BUFF_AWIDTH - 1 : 0] wr_addr,
                   input[g_AXI_DWIDTH - 1 : 0] wr_data_i,
                   output wire[g_AXI_DWIDTH - 1 : 0] rd_data_o);
  reg[g_AXI_BUFF_AWIDTH - 1 : 0] Ill;
  reg[g_AXI_DWIDTH - 1 : 0] lll[AXI_BUFF_DEPTH - 1 : 0];
  always
    @(posedge wclock)
    begin
      if (we)
        begin
          lll[wr_addr] <= wr_data_i;
        end
    end
  always
    @(posedge rclock)
    begin
      Ill <= rd_addr;
    end
  assign rd_data_o = lll[Ill];
endmodule
