/*****************************************************************************************************************************
--
--    File Name    : data_unpack_64_24.v

--    Description  : This module implements 64-bit to 24-bit unpacker for read channel 2, 3 and 4
--                   

-- Targeted device                     : Microsemi-SoC
-- Author                              : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module unpack_64_24 #(parameter g_AXI_DWIDTH = 64,
                      parameter g_AXI_BUFF_AWIDTH = 10,
                      parameter g_RD_CHANNEL_VIDEO_DATA_WIDTH = 24)
                    (input wire clock,
                     input wire nreset,
                     input wire start_unpack,
                     input wire[(g_AXI_BUFF_AWIDTH - 1) : 0] locs_to_read,
                     input wire[(g_AXI_DWIDTH - 1) : 0] buff_data_in,
                     output reg[(g_AXI_BUFF_AWIDTH - 1) : 0] buff_raddr,
                     output reg[(g_RD_CHANNEL_VIDEO_DATA_WIDTH - 1) : 0] video_data,
                     output reg data_valid,
                     output reg unpacker_compl);
  parameter[3 : 0] lol = 4'd0,
                   ool = 4'd1,
                   iol = 4'd2,
                   Oil = 4'd3,
                   Iil = 4'd4,
                   lil = 4'd5,
                   oil = 4'd6,
                   iil = 4'd7,
                   OO0 = 4'd8,
                   IO0 = 4'd9,
                   lO0 = 4'd10;
  reg[3 : 0] oO0;
  reg[15 : 0] ill;
  reg[(g_AXI_DWIDTH - 1) : 0] iO0;
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          iO0 <= { g_AXI_DWIDTH { 1'b0 } };
        end
      else
        begin
          iO0 <= buff_data_in;
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        oO0 <= lol;
      else
        begin
          case (oO0)
            lol:
              begin
                if (start_unpack)
                  oO0 <= ool;
                else
                  oO0 <= lol;
              end
            ool:
              begin
                oO0 <= iol;
              end
            iol:
              begin
                oO0 <= Oil;
              end
            Oil:
              begin
                oO0 <= Iil;
              end
            Iil:
              begin
                oO0 <= lil;
              end
            lil:
              begin
                oO0 <= oil;
              end
            oil:
              begin
                oO0 <= iil;
              end
            iil:
              begin
                oO0 <= OO0;
              end
            OO0:
              begin
                oO0 <= IO0;
              end
            IO0:
              begin
                oO0 <= lO0;
              end
            lO0:
              begin
                if (buff_raddr == locs_to_read)
                  oO0 <= lol;
                else
                  oO0 <= Oil;
              end
            default:
              oO0 <= lol;
          endcase
        end
    end
  always
    @(posedge clock,
      negedge nreset)
    begin
      if (!nreset)
        begin
          video_data <= { g_RD_CHANNEL_VIDEO_DATA_WIDTH { 1'b0 } };
          data_valid <= 1'b0;
          ill <= 16'd0;
          buff_raddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
          unpacker_compl <= 1'b0;
        end
      else
        begin
          case (oO0)
            lol:
              begin
                video_data <= { g_RD_CHANNEL_VIDEO_DATA_WIDTH { 1'b0 } };
                ill <= 16'd0;
                buff_raddr <= { g_AXI_BUFF_AWIDTH { 1'b0 } };
                data_valid <= 1'b0;
                unpacker_compl <= 1'b0;
              end
            ool:
              begin
                video_data <= { g_RD_CHANNEL_VIDEO_DATA_WIDTH { 1'b0 } };
              end
            iol:
              begin
                buff_raddr <= buff_raddr + 1'b1;
              end
            Oil:
              begin
                video_data <= iO0[23 : 0];
                data_valid <= 1'b1;
              end
            Iil:
              begin
                video_data <= iO0[47 : 24];
                data_valid <= 1'b1;
                ill <= iO0[63 : 48];
              end
            lil:
              begin
                video_data <= { iO0[7 : 0], ill };
                data_valid <= 1'b1;
                buff_raddr <= buff_raddr + 1'b1;
              end
            oil:
              begin
                video_data <= iO0[31 : 8];
                data_valid <= 1'b1;
              end
            iil:
              begin
                video_data <= iO0[55 : 32];
                data_valid <= 1'b1;
                ill <= { 8'd0, iO0[63 : 56] };
              end
            OO0:
              begin
                video_data <= { iO0[15 : 0], ill[7 : 0] };
                data_valid <= 1'b1;
                buff_raddr <= buff_raddr + 1'b1;
              end
            IO0:
              begin
                video_data <= iO0[39 : 16];
                data_valid <= 1'b1;
              end
            lO0:
              begin
                video_data <= iO0[63 : 40];
                data_valid <= 1'b1;
                if (buff_raddr == locs_to_read)
                  begin
                    unpacker_compl <= 1'b1;
                  end
                else
                  begin
                    unpacker_compl <= 1'b0;
                    buff_raddr <= buff_raddr + 1'b1;
                  end
              end
            default:
              begin
              end
          endcase
        end
    end
endmodule
