/*****************************************************************************************************************************
--
--    File Name    : ReadChannel_4_tb.v 

--    Description  : DDR Arbiter Read Channel 4 Test Bench
					 

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/


`timescale 1 ns / 1 ps
module Read_TB_4 
 #(
   parameter g_DATA_WIDTH = 24,
   parameter g_DDR_ADDRESS_WIDTH = 32,
   parameter g_WR_CHANNEL_AXI_BUFF_AWIDTH = 13,
   parameter g_RD_CHANNEL_AXI_BUFF_AWIDTH = 13
 )(
   input wire clock_i,
   input wire nreset_i,

input wire [23:0] video_rdata ,
input wire        rdata_valid ,
   
   input wire write_done_i,
   output wire [(g_DDR_ADDRESS_WIDTH-1):0] start_read_ddr_address_o,
   output reg [(g_WR_CHANNEL_AXI_BUFF_AWIDTH + 3 - 1):0]bytes_to_read_o,
   output reg read_req_o,
   input wire read_ack_i,
   input wire read_done_i,
   input wire [31:0] counter_v_rgb_in
 );


    parameter DDR_START_ADDRESS = 32'h8D000000;
    parameter HEIGHT=200;
    parameter WIDTH=320;
    parameter TOTAL_BYTES_TO_TRANSFER = WIDTH*3;
    parameter MAX_SIZE=(HEIGHT*WIDTH*3)+4;
    parameter MAX_SIZE_32=(HEIGHT*WIDTH)+1;

 reg [(g_WR_CHANNEL_AXI_BUFF_AWIDTH + 3 - 1):0]  bytes_counter, data_counter;
 reg data_counter_en, data_gen_done;
 reg [(g_DDR_ADDRESS_WIDTH-1):0] start_read_ddr_address_reg;

 assign start_read_ddr_address_o = start_read_ddr_address_reg;

 parameter [3:0]      
                 wait_for_write_done = 4'd0, 
                 gen_read_trans = 4'd1,
                 wait_for_read_ack = 4'd2,
                 wait_for_read_done = 4'd3,
                 check_num_of_bytes = 4'd4,
                 tb_compl = 4'd5;
  reg [3:0] beat_gen_tb_fsm;
  integer counter_v_rgb;

 always @(posedge clock_i or negedge nreset_i)
 begin
  if (!nreset_i)
  begin
   beat_gen_tb_fsm <= wait_for_write_done;
   bytes_counter <= TOTAL_BYTES_TO_TRANSFER;
   bytes_to_read_o <= {(g_WR_CHANNEL_AXI_BUFF_AWIDTH + 3) {1'b0}};
   read_req_o <= 1'b0;
   start_read_ddr_address_reg <= DDR_START_ADDRESS;
   counter_v_rgb <= 0;
  end
  else
  begin
    case (beat_gen_tb_fsm)
   
     wait_for_write_done    :begin
                              if (counter_v_rgb_in == HEIGHT-1)
                              begin                              
                                beat_gen_tb_fsm <= gen_read_trans;
                               
                              end
                              else
                              begin                               
                               beat_gen_tb_fsm <= wait_for_write_done;
                              end
                             end                             
     gen_read_trans         :begin
                              start_read_ddr_address_reg <= start_read_ddr_address_reg ;
                              bytes_to_read_o <= bytes_counter;
                              read_req_o <= 1'b1;
                              beat_gen_tb_fsm <= wait_for_read_ack;
                             end
     wait_for_read_ack     :begin
                              if (read_ack_i)
                              begin
                               read_req_o <= 1'b0;
                               beat_gen_tb_fsm <= wait_for_read_done;
                              end
                              else
                              begin
                               read_req_o <= 1'b1;
                               beat_gen_tb_fsm <= wait_for_read_ack;
                              end
                             end
     wait_for_read_done    :begin
                              if (read_done_i)
                              begin                              
                               beat_gen_tb_fsm <= check_num_of_bytes;
                                 counter_v_rgb <= counter_v_rgb+1;
                              end
                              else
                              begin                               
                               beat_gen_tb_fsm <= wait_for_read_done;
                              end
                             end                             
      check_num_of_bytes    :begin
                             if (counter_v_rgb == HEIGHT)
                              begin
                               start_read_ddr_address_reg <= DDR_START_ADDRESS;
                               beat_gen_tb_fsm <= tb_compl;
                              end
                              else
                              begin
                               start_read_ddr_address_reg <= start_read_ddr_address_reg + TOTAL_BYTES_TO_TRANSFER;
                                beat_gen_tb_fsm <= gen_read_trans;
                              end
                             end
     tb_compl               :beat_gen_tb_fsm <= tb_compl;
                             
    endcase
  end
 end

/**************************
Write image data read into file
**************************/
integer channel_file_handle =0;
initial 
begin
	channel_file_handle = $fopen("Read_out_rd_ch4.txt","wb");
	$fwrite(channel_file_handle,"Resolution\nHorizontal: %0d\nVertical: %0d\n",WIDTH,HEIGHT);
end

always @ (posedge clock_i)
begin
	if (rdata_valid == 1)begin
		$fwrite(channel_file_handle,"%x\n",video_rdata[23:16]);
		$fwrite(channel_file_handle,"%x\n",video_rdata[15:08]);
		$fwrite(channel_file_handle,"%x\n",video_rdata[07:00]);
	end
end
/**************************
Close file after writing vertical resolution lines of data
**************************/
always @ (posedge clock_i)
begin
    if (counter_v_rgb == HEIGHT)
    begin
        $fclose(channel_file_handle);
    end
    if (counter_v_rgb == HEIGHT-1)
    begin
        $display("READ CHANNEL 4: FRAME READ COMPLETE IN NEXT TRANSACTION");
    end
end


 endmodule
