/*****************************************************************************************************************************
--
--    File Name    : byte2pixel_conversion.v 

--    Description  : This module implements conversion of byte data to pixel data.


-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
module byte2pixel_conversion #(parameter g_DATAWIDTH = 8,
                               parameter g_LANE_WIDTH = 4)
                             (input CAM_CLOCK_I,
                              input READ_CLOCK_I,
                              input RESET_n_I,
                              input wire[g_LANE_WIDTH * g_DATAWIDTH - 1 : 0] byte_data_i,
                              input wire byte_en_i,
                              output reg[g_DATAWIDTH - 1 : 0] pixel_data_o,
                              output reg line_valid_o,
                              output wire line_start_o,
                              output wire line_end_o);
  parameter OI = 2'b00,
            II = 2'b01,
            lI = 2'b10,
            oI = 2'b11;
  reg iI;
  reg Ol;
  reg[g_LANE_WIDTH * g_DATAWIDTH - 1 : 0] Il,
                                          ll,
                                          ol,
                                          il;
  wire[g_LANE_WIDTH * g_DATAWIDTH - 1 : 0] O0;
  reg I0,
      l0,
      o0;
  reg[3 : 0] i0;
  reg[4 : 0] O1;
  wire I1,
       l1;
  reg o1,
      i1;
  reg[1 : 0] Oo,
             Io;
  parameter lo = (g_DATAWIDTH == 8) ? 8 * g_LANE_WIDTH : 40;
  always
    @(posedge CAM_CLOCK_I or
      negedge RESET_n_I)
    if (!RESET_n_I)
      begin
        Il <= 0;
        o0 <= 0;
      end
    else
      begin
        Il <= byte_data_i;
        o0 <= byte_en_i;
      end
  generate
    if (g_DATAWIDTH == 8)
      begin
        always
          @(posedge CAM_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            begin
              ll <= 0;
              I0 <= 0;
            end
          else
            begin
              ll <= Il;
              I0 <= o0;
            end
      end
  endgenerate
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_n_I)
    if (!RESET_n_I)
      i0 <= 0;
    else
      if (I1)
        i0 <= 0;
      else
        if (i0 <= 8)
          i0 <= i0 + 1;
        else
          i0 <= i0;
  generate
    if ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 1))
      begin
        always
          @(posedge READ_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            l0 <= 0;
          else
            if (i0 > 8)
              l0 <= ~I1;
            else
              l0 <= 0;
      end
    else
      if ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 2))
        begin
          always
            @(posedge READ_CLOCK_I or
              negedge RESET_n_I)
            if (!RESET_n_I)
              begin
                l0 <= 0;
                i1 <= 0;
              end
            else
              if (i0 > 8)
                begin
                  i1 <= i1 + 1;
                  l0 <= ~i1;
                end
              else
                begin
                  l0 <= 0;
                  i1 <= 0;
                end
        end
      else
        if ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 4))
          begin
            always
              @(posedge READ_CLOCK_I or
                negedge RESET_n_I)
              if (!RESET_n_I)
                begin
                  l0 <= 0;
                  Io <= 0;
                end
              else
                if (i0 > 8)
                  begin
                    Io <= Io + 1;
                    l0 <= ~Io[0] & ~Io[1];
                  end
                else
                  begin
                    l0 <= 0;
                    Io <= 0;
                  end
          end
  endgenerate
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        O1 <= 0;
      else
        O1 <= { O1[3 : 0], l0 };
    end
  assign l1 = ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 1)) ? O1[0] : ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 2)) ? (O1[0] | O1[1]) : ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 4)) ? (O1[0] | O1[1] | O1[2] | l0) : O1[0];
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        line_valid_o <= 0;
      else
        line_valid_o <= (l1) ? 1'b1 : 1'b0;
    end
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          ol <= 0;
          il <= 0;
        end
      else
        begin
          ol <= O0;
          il <= ol;
        end
    end
  generate
    if ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 1))
      begin
        always
          @(posedge READ_CLOCK_I or
            negedge RESET_n_I)
          if (!RESET_n_I)
            begin
              pixel_data_o <= 0;
            end
          else
            if (l1)
              begin
                pixel_data_o <= ol;
              end
            else
              begin
                pixel_data_o <= 0;
              end
      end
    else
      if ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 2))
        begin
          always
            @(posedge READ_CLOCK_I or
              negedge RESET_n_I)
            if (!RESET_n_I)
              begin
                o1 <= 0;
                pixel_data_o <= 0;
              end
            else
              if (l1)
                begin
                  o1 <= o1 + 1;
                  pixel_data_o <= (o1) ? ol[15 : 8] : ol[7 : 0];
                end
              else
                begin
                  o1 <= 0;
                  pixel_data_o <= 0;
                end
        end
      else
        if ((g_DATAWIDTH == 8) && (g_LANE_WIDTH == 4))
          begin
            always
              @(posedge READ_CLOCK_I or
                negedge RESET_n_I)
              if (!RESET_n_I)
                begin
                  Oo <= 0;
                  pixel_data_o <= 0;
                end
              else
                if (l1)
                  begin
                    case (Oo)
                      OI:
                        begin
                          pixel_data_o <= il[7 : 0];
                          Oo <= II;
                        end
                      II:
                        begin
                          pixel_data_o <= il[15 : 8];
                          Oo <= lI;
                        end
                      lI:
                        begin
                          pixel_data_o <= il[23 : 16];
                          Oo <= oI;
                        end
                      oI:
                        begin
                          pixel_data_o <= il[31 : 24];
                          Oo <= OI;
                        end
                      default:
                        pixel_data_o <= 0;
                    endcase
                  end
                else
                  begin
                    Oo <= 0;
                    pixel_data_o <= 0;
                  end
          end
  endgenerate
  cdcfifo #(.ADDR_WIDTH(9), .DATA_WIDTH(lo)) oo(.k_lim(12'h128),
                                                 .wrrstn(RESET_n_I),
                                                 .wrclk(CAM_CLOCK_I),
                                                 .wrreq(I0),
                                                 .wrdata(ll),
                                                 .wrfull(),
                                                 .wrusedw(),
                                                 .rdrstn(RESET_n_I),
                                                 .rdclk(READ_CLOCK_I),
                                                 .rdreq(l0),
                                                 .rddata(O0),
                                                 .rdempty(I1),
                                                 .rdusedw());
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_n_I)
    if (!RESET_n_I)
      iI <= 0;
    else
      iI <= l1;
  always
    @(posedge READ_CLOCK_I or
      negedge RESET_n_I)
    if (!RESET_n_I)
      begin
        Ol <= 0;
      end
    else
      begin
        Ol <= iI;
      end
  assign line_start_o = l1 & ~iI;
  assign line_end_o = ~iI & Ol;
endmodule
