/*****************************************************************************************************************************
--
--    File Name    : mipicsi2rxdecoderPF.v 

--    Description  : This is the top level module of MIPI CSI-2 interface.


-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2018 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/100ps
module mipicsi2rxdecoderPF #(parameter g_DATAWIDTH = 8,
                                parameter g_LANE_WIDTH = 4,
                                parameter g_INPUT_DATA_INVERT = 0,
                                parameter g_BUFF_DEPTH = 1280)
                              (input CAM_CLOCK_I,
                               input PARALLEL_CLOCK_I,
                               input RESET_n_I,
                               input[7 : 0] L0_HS_DATA_I,
                               input[7 : 0] L1_HS_DATA_I,
                               input[7 : 0] L2_HS_DATA_I,
                               input[7 : 0] L3_HS_DATA_I,
                               input L0_LP_DATA_I,
                               input L0_LP_DATA_N_I,
                               input L1_LP_DATA_I,
                               input L1_LP_DATA_N_I,
                               input L2_LP_DATA_I,
                               input L2_LP_DATA_N_I,
                               input L3_LP_DATA_I,
                               input L3_LP_DATA_N_I,
                               output[7 : 0] data_out_o,
                               output frame_end_o,
                               output frame_start_o,
                               output line_end_o,
                               output line_start_o,
                               output line_valid_o,
                               output[15 : 0] word_count_o);
  wire[7 : 0] i0OI;
  wire[7 : 0] O1OI;
  wire[7 : 0] I1OI;
  wire[7 : 0] l1OI;
  embsync_detect #(.g_LANE_WIDTH(g_LANE_WIDTH)) o1OI(.l1l(RESET_n_I),
                                                     .CLOCK_I(CAM_CLOCK_I),
                                                     .HS(RESET_n_I),
                                                     .L0_DATA_I(L0_HS_DATA_I),
                                                     .L1_DATA_I(L1_HS_DATA_I),
                                                     .L2_DATA_I(L2_HS_DATA_I),
                                                     .L3_DATA_I(L3_HS_DATA_I),
                                                     .L0_DATA_O(i0OI),
                                                     .L1_DATA_O(O1OI),
                                                     .L2_DATA_O(I1OI),
                                                     .L3_DATA_O(l1OI));
  mipi_csi2_rxdecoder #(.g_DATAWIDTH(g_DATAWIDTH), .g_INPUT_DATA_INVERT(g_INPUT_DATA_INVERT), .g_LANE_WIDTH(g_LANE_WIDTH), .g_BUFF_DEPTH(g_BUFF_DEPTH)) i1OI(.CAM_CLOCK_I(CAM_CLOCK_I),
                                                                                                                                                             .READ_CLOCK_I(PARALLEL_CLOCK_I),
                                                                                                                                                             .RESET_n_I(RESET_n_I),
                                                                                                                                                             .ChannelA_i(i0OI),
                                                                                                                                                             .ChannelB_i(O1OI),
                                                                                                                                                             .ChannelC_i(I1OI),
                                                                                                                                                             .ChannelD_i(l1OI),
                                                                                                                                                             .pixel_valid_o(line_valid_o),
                                                                                                                                                             .frame_start_o(frame_start_o),
                                                                                                                                                             .frame_end_o(frame_end_o),
                                                                                                                                                             .line_start_o(line_start_o),
                                                                                                                                                             .line_end_o(line_end_o),
                                                                                                                                                             .data_out_o(data_out_o),
                                                                                                                                                             .word_count_o(word_count_o),
                                                                                                                                                             .frame_number_o());
endmodule
