/*****************************************************************************************************************************
--
--    File Name    : Delay.v

--    Description  : This module implements delay block                                   

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team


-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/
`timescale 1ns/1ps
module Delay #(
/******************************************************************************************************************************/
/***********************************************Parameters*********************************************************************/
/******************************************************************************************************************************/            
                          parameter g_DATAWIDTH = 24,
                          parameter g_X_RES_WIDTH = 11,
                          parameter g_DISPLAY_RESOLUTION = 1280,
                          parameter g_VERT_DISPLAY_RESOLUTION = 720)
                        (
/******************************************************************************************************************************/
/***********************************************Module IO**********************************************************************/
/******************************************************************************************************************************/
                         input wire SYS_CLK_I,
                         input wire RESET_n_I,
                         input wire[(g_DATAWIDTH ) - 1 : 0] Y_in_i,
                         input wire Data_In_Vld_i,
                         output reg[(g_DATAWIDTH ) - 1 : 0] Y_Out_o,
                         output reg data_Vld_out_o
                        );



  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_write_addr;
  reg[g_DATAWIDTH - 1 : 0] s_r_data_in;
  reg Data_In_Vld_d;
  reg[2 : 0] s_r_line_Count;
  reg[g_X_RES_WIDTH - 1 : 0] in_line_Count;
  reg s_start_read;
  reg s_r_we0;
  reg s_r_we1;
  reg s_r_we2;
  reg s_r_we3;
  reg s_r_we4;
  reg s_r_re0;
  reg s_r_re1;
  reg s_r_re2;
  reg s_r_re3;
  reg s_r_re4;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr0;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr1;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr2;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr3;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr4;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr0_reg;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr1_reg;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr2_reg;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr3_reg;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_line_read_addr4_reg;
  reg[g_DATAWIDTH - 1 : 0] s_r_p22_i;
  reg[g_DATAWIDTH - 1 : 0] s_r_p23_i;
  reg[g_DATAWIDTH - 1 : 0] s_r_p24_i;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d1;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d2;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d3;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d4;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d5;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d6;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d7;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d8;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d9;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d10;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d11;
  reg[(g_DATAWIDTH ) - 1 : 0] s_r_p22_MedIn22_d12;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_Outpixel_cnt;
  reg[g_X_RES_WIDTH - 1 : 0] s_r_Outline_cnt;
  wire[g_DATAWIDTH - 1 : 0] s_r_read_data0;
  wire[g_DATAWIDTH - 1 : 0] s_r_read_data1;
  wire[g_DATAWIDTH - 1 : 0] s_r_read_data2;
  wire[g_DATAWIDTH - 1 : 0] s_r_read_data3;
  wire[g_DATAWIDTH - 1 : 0] s_r_read_data4;
  reg[g_DATAWIDTH - 1 : 0] s_r_read_data0_reg;
  reg[g_DATAWIDTH - 1 : 0] s_r_read_data1_reg;
  reg[g_DATAWIDTH - 1 : 0] s_r_read_data2_reg;
  reg[g_DATAWIDTH - 1 : 0] s_r_read_data3_reg;
  reg[g_DATAWIDTH - 1 : 0] s_r_read_data4_reg;
  reg[2 : 0] s_r_rd_cnt;
  reg[2 : 0] s_r_rd_cnt_reg;
  reg extd_flag;
  reg[g_X_RES_WIDTH - 1 : 0] output_pix_cntr;
  parameter[0 : 0] cntr_start = 1'b0,
                   cntr_run = 1'b1;
  reg[0 : 0] out_cntr_fsm;
  reg data_Vld_out;
  reg data_Vld_out_d;
  reg data_Vld_out_d1;
  reg data_Vld_out_d2;
  reg data_Vld_out_d3;
  reg data_Vld_out_d4;
  reg data_Vld_out_d5;
  reg data_Vld_out_d6;
  reg data_Vld_out_d7;
  reg data_Vld_out_d8;
  reg data_Vld_out_d9;
  reg data_Vld_out_d10;
  reg data_Vld_out_d11;
  reg data_Vld_out_d12;
  parameter[2 : 0] idle = 3'h0,
                   check_line_compl = 3'h1,
                   check_input_data = 3'h2,
                   check_frame_compl = 3'h3,
                   last_line = 3'h4,
                   last_line_written = 3'h5;
  reg[2 : 0] start_read_fsm;
  reg s_read_hold;
  reg last_line_written_flag;
  reg read_ack;




/******************************************************************************************************************************/
/***************************Process to generate input line buffer write address************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_line_write_addr <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if (s_r_line_write_addr == g_DISPLAY_RESOLUTION - 1)
            begin
              s_r_line_write_addr <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (Data_In_Vld_d == 1'b1)
                begin
                  s_r_line_write_addr <= s_r_line_write_addr + 1'b1;
                end
            end
        end
    end



/******************************************************************************************************************************/
/***************************Process to generate input line Count***************************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_line_Count <= 3'b000;
        end
      else
        begin
          if ((s_r_line_Count == 3'b100 && s_r_line_write_addr == g_DISPLAY_RESOLUTION - 1) || (in_line_Count == g_VERT_DISPLAY_RESOLUTION - 1 && s_r_line_write_addr == g_DISPLAY_RESOLUTION - 1))
            begin
              s_r_line_Count <= 3'b000;
            end
          else
            begin
              if (s_r_line_write_addr == g_DISPLAY_RESOLUTION - 1)
                begin
                  s_r_line_Count <= s_r_line_Count + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          in_line_Count <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if (in_line_Count == g_VERT_DISPLAY_RESOLUTION)
            begin
              in_line_Count <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (s_r_line_write_addr == g_DISPLAY_RESOLUTION - 1 && Data_In_Vld_d == 1'b1)
                begin
                  in_line_Count <= in_line_Count + 1'b1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          last_line_written_flag <= 1'b0;
        end
      else
        begin
          if (in_line_Count == g_VERT_DISPLAY_RESOLUTION)
            begin
              last_line_written_flag <= 1'b1;
            end
          else
            if (read_ack)
              begin
                last_line_written_flag <= 1'b0;
              end
        end
    end




/******************************************************************************************************************************/
/***************************Process to register input pixels*******************************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_data_in <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
           s_r_data_in <= { Y_in_i};
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Data_In_Vld_d <= 1'b0;
        end
      else
        begin
          Data_In_Vld_d <= Data_In_Vld_i;
        end
    end




/******************************************************************************************************************************/
/**************************Process to generate the line buffer write enables***************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_we0 <= 1'b0;
          s_r_we1 <= 1'b0;
          s_r_we2 <= 1'b0;
          s_r_we3 <= 1'b0;
          s_r_we4 <= 1'b0;
        end
      else
        if (Data_In_Vld_i)
          begin
            case (s_r_line_Count)
              3'd0:
                begin
                  s_r_we0 <= 1'b1;
                  s_r_we1 <= 1'b0;
                  s_r_we2 <= 1'b0;
                  s_r_we3 <= 1'b0;
                  s_r_we4 <= 1'b0;
                end
              3'd1:
                begin
                  s_r_we0 <= 1'b0;
                  s_r_we1 <= 1'b1;
                  s_r_we2 <= 1'b0;
                  s_r_we3 <= 1'b0;
                  s_r_we4 <= 1'b0;
                end
              3'd2:
                begin
                  s_r_we0 <= 1'b0;
                  s_r_we1 <= 1'b0;
                  s_r_we2 <= 1'b1;
                  s_r_we3 <= 1'b0;
                  s_r_we4 <= 1'b0;
                end
              3'd3:
                begin
                  s_r_we0 <= 1'b0;
                  s_r_we1 <= 1'b0;
                  s_r_we2 <= 1'b0;
                  s_r_we3 <= 1'b1;
                  s_r_we4 <= 1'b0;
                end
              3'd4:
                begin
                  s_r_we0 <= 1'b0;
                  s_r_we1 <= 1'b0;
                  s_r_we2 <= 1'b0;
                  s_r_we3 <= 1'b0;
                  s_r_we4 <= 1'b1;
                end
              default:
                begin
                  s_r_we0 <= 1'b0;
                  s_r_we1 <= 1'b0;
                  s_r_we2 <= 1'b0;
                  s_r_we3 <= 1'b0;
                  s_r_we4 <= 1'b0;
                end
            endcase
          end
        else
          begin
            s_r_we0 <= 1'b0;
            s_r_we1 <= 1'b0;
            s_r_we2 <= 1'b0;
            s_r_we3 <= 1'b0;
            s_r_we4 <= 1'b0;
          end
    end






/******************************************************************************************************************************/
/***************************Process to register input pixels*******************************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_start_read <= 1'b0;
        end
      else
        begin
          if ((s_r_line_Count >= 3'b010) && (s_r_line_write_addr >= 11'h004))
            begin
              s_start_read <= 1'b1;
            end
          else
            if ((s_r_Outline_cnt == g_VERT_DISPLAY_RESOLUTION - 1) && (s_r_Outpixel_cnt == g_DISPLAY_RESOLUTION - 1))
              begin
                s_start_read <= 1'b0;
              end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          start_read_fsm <= idle;
        end
      else
        begin
          case (start_read_fsm)
            idle:
              begin
                if (s_r_line_Count >= 3'b010 && s_r_line_write_addr >= 11'h004)
                  begin
                    start_read_fsm <= check_line_compl;
                  end
                else
                  begin
                    start_read_fsm <= idle;
                  end
              end
            check_line_compl:
              begin
                if (s_r_Outpixel_cnt == g_DISPLAY_RESOLUTION)
                  begin
                    start_read_fsm <= last_line_written;
                  end
                else
                  begin
                    start_read_fsm <= check_line_compl;
                  end
              end
            last_line_written:
              begin
                if (last_line_written_flag)
                  begin
                    start_read_fsm <= check_frame_compl;
                  end
                else
                  begin
                    start_read_fsm <= check_input_data;
                  end
              end
            check_input_data:
              begin
                if (Data_In_Vld_i && s_r_line_write_addr >= 11'h004)
                  begin
                    start_read_fsm <= check_frame_compl;
                  end
                else
                  begin
                    start_read_fsm <= check_input_data;
                  end
              end
            check_frame_compl:
              begin
                if (s_r_Outline_cnt == g_VERT_DISPLAY_RESOLUTION - 1)
                  begin
                    start_read_fsm <= last_line;
                  end
                else
                  begin
                    start_read_fsm <= check_line_compl;
                  end
              end
            last_line:
              begin
                if (s_r_Outpixel_cnt == g_DISPLAY_RESOLUTION - 1)
                  begin
                    start_read_fsm <= idle;
                  end
                else
                  begin
                    start_read_fsm <= last_line;
                  end
              end
            default:
              start_read_fsm <= idle;
          endcase
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_read_hold <= 1'b0;
          read_ack <= 1'b0;
        end
      else
        begin
          case (start_read_fsm)
            idle:
              begin
                s_read_hold <= 1'b0;
                read_ack <= 1'b0;
              end
            check_line_compl:
              begin
                if (s_r_Outpixel_cnt == g_DISPLAY_RESOLUTION)
                  begin
                    s_read_hold <= 1'b1;
                  end
                else
                  begin
                    s_read_hold <= 1'b0;
                  end
              end
            last_line_written:
              begin
                s_read_hold <= 1'b1;
              end
            check_input_data:
              begin
                if (Data_In_Vld_i && s_r_line_write_addr >= 11'h004)
                  begin
                    s_read_hold <= 1'b0;
                  end
                else
                  begin
                    s_read_hold <= s_read_hold;
                  end
              end
            check_frame_compl:
              begin
                s_read_hold <= 1'b0;
              end
            last_line:
              begin
                if (s_r_Outpixel_cnt == g_DISPLAY_RESOLUTION - 1)
                  begin
                    s_read_hold <= 1'b0;
                    read_ack <= 1'b1;
                  end
                else
                  begin
                    s_read_hold <= s_read_hold;
                    read_ack <= 1'b0;
                  end
              end
            default:
              begin
                s_read_hold <= 1'b0;
                read_ack <= 1'b0;
              end
          endcase
        end
    end




/******************************************************************************************************************************/
/***************************Process to Output Pixel Count in each line*********************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_Outpixel_cnt <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (s_start_read == 1'b0 || s_r_Outpixel_cnt > g_DISPLAY_RESOLUTION - 1)
            begin
              s_r_Outpixel_cnt <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            if (s_start_read == 1'b1)
              begin
                if (s_read_hold)
                  s_r_Outpixel_cnt <= { g_X_RES_WIDTH { 1'b0 } };
                else
                  s_r_Outpixel_cnt <= s_r_Outpixel_cnt + 1'b1;
              end
        end
    end



/******************************************************************************************************************************/
/***************************Process to Pixel Count in each line****************************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_Outline_cnt <= { g_X_RES_WIDTH { 1'b1 } };
        end
      else
        begin
          if (s_start_read == 1'b0)
            begin
              s_r_Outline_cnt <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (s_r_Outpixel_cnt > g_DISPLAY_RESOLUTION - 1 && s_start_read == 1'b1)
                begin
                  s_r_Outline_cnt <= s_r_Outline_cnt + 1'b1;
                end
            end
        end
    end




/******************************************************************************************************************************/
/***************************Process to Pixel Count in each line****************************************************************/
/******************************************************************************************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_rd_cnt <= 3'd0;
        end
      else
        begin
          if (s_start_read == 1'b0 || (s_r_rd_cnt == 3'd4 && s_r_Outpixel_cnt == g_DISPLAY_RESOLUTION))
            begin
              s_r_rd_cnt <= 3'd0;
            end
          else
            begin
              if (s_r_Outpixel_cnt >= g_DISPLAY_RESOLUTION && s_start_read == 1'b1)
                begin
                  s_r_rd_cnt <= s_r_rd_cnt + 12'd1;
                end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_rd_cnt_reg <= 3'd0;
        end
      else
        begin
          s_r_rd_cnt_reg <= s_r_rd_cnt;
        end
    end




/******************************************************************************************************************************/
/***************************Process to generate the line buffer Read enables***************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_re0 <= 1'b1;
          s_r_re1 <= 1'b1;
          s_r_re2 <= 1'b1;
          s_r_re3 <= 1'b1;
          s_r_re4 <= 1'b1;
        end
      else
        begin
          case (s_r_rd_cnt_reg)
            3'd0:
              begin
                s_r_re0 <= 1'b0;
                s_r_re1 <= 1'b0;
                s_r_re2 <= 1'b0;
                s_r_re3 <= 1'b1;
                s_r_re4 <= 1'b1;
              end
            3'd1:
              begin
                s_r_re0 <= 1'b1;
                s_r_re1 <= 1'b0;
                s_r_re2 <= 1'b0;
                s_r_re3 <= 1'b0;
                s_r_re4 <= 1'b1;
              end
            3'd2:
              begin
                s_r_re0 <= 1'b1;
                s_r_re1 <= 1'b1;
                s_r_re2 <= 1'b0;
                s_r_re3 <= 1'b0;
                s_r_re4 <= 1'b0;
              end
            3'd3:
              begin
                s_r_re0 <= 1'b0;
                s_r_re1 <= 1'b1;
                s_r_re2 <= 1'b1;
                s_r_re3 <= 1'b0;
                s_r_re4 <= 1'b0;
              end
            3'd4:
              begin
                s_r_re0 <= 1'b0;
                s_r_re1 <= 1'b0;
                s_r_re2 <= 1'b1;
                s_r_re3 <= 1'b1;
                s_r_re4 <= 1'b0;
              end
            default:
              begin
                s_r_re0 <= 1'b0;
                s_r_re1 <= 1'b0;
                s_r_re2 <= 1'b0;
                s_r_re3 <= 1'b0;
                s_r_re4 <= 1'b0;
              end
          endcase
        end
    end






/******************************************************************************************************************************/
/***************************Process to generate the line buffer read addresses*************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_line_read_addr0 <= { g_X_RES_WIDTH { 1'b0 } };
          s_r_line_read_addr1 <= { g_X_RES_WIDTH { 1'b0 } };
          s_r_line_read_addr2 <= { g_X_RES_WIDTH { 1'b0 } };
          s_r_line_read_addr3 <= { g_X_RES_WIDTH { 1'b0 } };
          s_r_line_read_addr4 <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          if ((s_start_read == 1'b1 && (s_r_line_read_addr0 > g_DISPLAY_RESOLUTION - 1 || s_r_line_read_addr1 > g_DISPLAY_RESOLUTION - 1 || s_r_line_read_addr2 > g_DISPLAY_RESOLUTION - 1 || s_r_line_read_addr3 > g_DISPLAY_RESOLUTION - 1 || s_r_line_read_addr4 > g_DISPLAY_RESOLUTION - 1)))
            begin
              s_r_line_read_addr0 <= { g_X_RES_WIDTH { 1'b0 } };
              s_r_line_read_addr1 <= { g_X_RES_WIDTH { 1'b0 } };
              s_r_line_read_addr2 <= { g_X_RES_WIDTH { 1'b0 } };
              s_r_line_read_addr3 <= { g_X_RES_WIDTH { 1'b0 } };
              s_r_line_read_addr4 <= { g_X_RES_WIDTH { 1'b0 } };
            end
          else
            begin
              if (s_start_read == 1'b1)
                begin
                  if (s_read_hold == 1'b1)
                    begin
                      s_r_line_read_addr0 <= s_r_line_read_addr0;
                      s_r_line_read_addr1 <= s_r_line_read_addr1;
                      s_r_line_read_addr2 <= s_r_line_read_addr2;
                      s_r_line_read_addr3 <= s_r_line_read_addr3;
                      s_r_line_read_addr4 <= s_r_line_read_addr4;
                    end
                  else
                    begin
                      case (s_r_rd_cnt_reg)
                        3'd0:
                          begin
                            s_r_line_read_addr0 <= s_r_line_read_addr0 + 1'b1;
                            s_r_line_read_addr1 <= s_r_line_read_addr1 + 1'b1;
                            s_r_line_read_addr2 <= s_r_line_read_addr2 + 1'b1;
                          end
                        3'd1:
                          begin
                            s_r_line_read_addr1 <= s_r_line_read_addr1 + 1'b1;
                            s_r_line_read_addr2 <= s_r_line_read_addr2 + 1'b1;
                            s_r_line_read_addr3 <= s_r_line_read_addr3 + 1'b1;
                          end
                        3'd2:
                          begin
                            s_r_line_read_addr2 <= s_r_line_read_addr2 + 1'b1;
                            s_r_line_read_addr3 <= s_r_line_read_addr3 + 1'b1;
                            s_r_line_read_addr4 <= s_r_line_read_addr4 + 1'b1;
                          end
                        3'd3:
                          begin
                            s_r_line_read_addr3 <= s_r_line_read_addr3 + 1'b1;
                            s_r_line_read_addr4 <= s_r_line_read_addr4 + 1'b1;
                            s_r_line_read_addr0 <= s_r_line_read_addr0 + 1'b1;
                          end
                        3'd4:
                          begin
                            s_r_line_read_addr4 <= s_r_line_read_addr4 + 1'b1;
                            s_r_line_read_addr0 <= s_r_line_read_addr0 + 1'b1;
                            s_r_line_read_addr1 <= s_r_line_read_addr1 + 1'b1;
                          end
                        default:
                          begin
                            s_r_line_read_addr0 <= s_r_line_read_addr0;
                            s_r_line_read_addr1 <= s_r_line_read_addr1;
                            s_r_line_read_addr2 <= s_r_line_read_addr2;
                            s_r_line_read_addr3 <= s_r_line_read_addr3;
                            s_r_line_read_addr4 <= s_r_line_read_addr4;
                          end
                      endcase
                    end
                end
              else
                if (s_start_read == 1'b0)
                  begin
                    s_r_line_read_addr0 <= { g_X_RES_WIDTH { 1'b0 } };
                    s_r_line_read_addr1 <= { g_X_RES_WIDTH { 1'b0 } };
                    s_r_line_read_addr2 <= { g_X_RES_WIDTH { 1'b0 } };
                    s_r_line_read_addr3 <= { g_X_RES_WIDTH { 1'b0 } };
                    s_r_line_read_addr4 <= { g_X_RES_WIDTH { 1'b0 } };
                  end
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_line_read_addr0_reg <= { g_X_RES_WIDTH { 1'b0 } };
          s_r_line_read_addr1_reg <= { g_X_RES_WIDTH { 1'b0 } };
          s_r_line_read_addr2_reg <= { g_X_RES_WIDTH { 1'b0 } };
          s_r_line_read_addr3_reg <= { g_X_RES_WIDTH { 1'b0 } };
          s_r_line_read_addr4_reg <= { g_X_RES_WIDTH { 1'b0 } };
        end
      else
        begin
          s_r_line_read_addr0_reg <= s_r_line_read_addr0;
          s_r_line_read_addr1_reg <= s_r_line_read_addr1;
          s_r_line_read_addr2_reg <= s_r_line_read_addr2;
          s_r_line_read_addr3_reg <= s_r_line_read_addr3;
          s_r_line_read_addr4_reg <= s_r_line_read_addr4;
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_read_data0_reg <= { g_DATAWIDTH { 1'b0 } };
          s_r_read_data1_reg <= { g_DATAWIDTH { 1'b0 } };
          s_r_read_data2_reg <= { g_DATAWIDTH { 1'b0 } };
          s_r_read_data3_reg <= { g_DATAWIDTH { 1'b0 } };
          s_r_read_data4_reg <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          s_r_read_data0_reg <= s_r_read_data0;
          s_r_read_data1_reg <= s_r_read_data1;
          s_r_read_data2_reg <= s_r_read_data2;
          s_r_read_data3_reg <= s_r_read_data3;
          s_r_read_data4_reg <= s_r_read_data4;
        end
    end






/******************************************************************************************************************************/
/***************************Process to register Second column of 4x4 window****************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_p22_i <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (s_r_line_read_addr2_reg == 11'd2 || s_r_line_read_addr3_reg == 11'd2 || s_r_line_read_addr4_reg == 11'd2)
            begin
              case (s_r_rd_cnt_reg)
                3'd0:
                  begin
                    s_r_p22_i <= s_r_read_data1_reg;
                  end
                3'd1:
                  begin
                    s_r_p22_i <= s_r_read_data2_reg;
                  end
                3'd2:
                  begin
                    s_r_p22_i <= s_r_read_data3_reg;
                  end
                3'd3:
                  begin
                    s_r_p22_i <= s_r_read_data4_reg;
                  end
                3'd4:
                  begin
                    s_r_p22_i <= s_r_read_data0_reg;
                  end
                default:
                  begin
                    s_r_p22_i <= s_r_p22_i;
                  end
              endcase
            end
          else
            begin
              if (s_r_line_read_addr2_reg >= 11'd5 || s_r_line_read_addr3_reg >= 11'd5 || s_r_line_read_addr4_reg >= 11'd5)
                begin
                  s_r_p22_i <= s_r_p23_i;
                end
            end
        end
    end




/******************************************************************************************************************************/
/***************************Process to register third column of 4x4 window*****************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_p23_i <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (s_r_line_read_addr2_reg == 11'd3 || s_r_line_read_addr3_reg == 11'd3 || s_r_line_read_addr4_reg == 11'd3)
            begin
              case (s_r_rd_cnt_reg)
                3'd0:
                  begin
                    s_r_p23_i <= s_r_read_data1_reg;
                  end
                3'd1:
                  begin
                    s_r_p23_i <= s_r_read_data2_reg;
                  end
                3'd2:
                  begin
                    s_r_p23_i <= s_r_read_data3_reg;
                  end
                3'd3:
                  begin
                    s_r_p23_i <= s_r_read_data4_reg;
                  end
                3'd4:
                  begin
                    s_r_p23_i <= s_r_read_data0_reg;
                  end
                default:
                  begin
                    s_r_p23_i <= s_r_p23_i;
                  end
              endcase
            end
          else
            begin
              if (s_r_line_read_addr2_reg >= 11'd5 || s_r_line_read_addr3_reg >= 11'd5 || s_r_line_read_addr4_reg >= 11'd5)
                begin
                  s_r_p23_i <= s_r_p24_i;
                end
            end
        end
    end




/******************************************************************************************************************************/
/*************************** Process to register Fourth column of 4x4 window***************************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_p24_i <= { g_DATAWIDTH { 1'b0 } };
        end
      else
        begin
          if (s_r_line_read_addr2_reg >= 11'd4 || s_r_line_read_addr3_reg >= 11'd4 || s_r_line_read_addr4_reg >= 11'd4)
            begin
              case (s_r_rd_cnt_reg)
                3'd0:
                  begin
                    s_r_p24_i <= s_r_read_data1_reg;
                  end
                3'd1:
                  begin
                    s_r_p24_i <= s_r_read_data2_reg;
                  end
                3'd2:
                  begin
                    s_r_p24_i <= s_r_read_data3_reg;
                  end
                3'd3:
                  begin
                    s_r_p24_i <= s_r_read_data4_reg;
                  end
                3'd4:
                  begin
                    s_r_p24_i <= s_r_read_data0_reg;
                  end
                default:
                  begin;
                    s_r_p24_i <= s_r_p24_i;
                  end
              endcase
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          extd_flag <= 1'b0;
        end
      else
        begin
          if (s_r_line_read_addr0_reg == g_DISPLAY_RESOLUTION || s_r_line_read_addr1_reg == g_DISPLAY_RESOLUTION || s_r_line_read_addr2_reg == g_DISPLAY_RESOLUTION || s_r_line_read_addr3_reg == g_DISPLAY_RESOLUTION || s_r_line_read_addr4_reg == g_DISPLAY_RESOLUTION)
            begin
              extd_flag <= 1'b1;
            end
          else
            if (s_r_line_read_addr0_reg <= 11'd4 || s_r_line_read_addr1_reg >= 11'd4 || s_r_line_read_addr2_reg >= 11'd4 || s_r_line_read_addr3_reg >= 11'd4 || s_r_line_read_addr4_reg >= 11'd4)
              begin
                extd_flag <= 1'b0;
              end
        end
    end





/******************************************************************************************************************************/
/***************************Registering input pixels for bilinear interpolation module*****************************************/
/******************************************************************************************************************************/
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_p22_MedIn22 <= { (g_DATAWIDTH) { 1'b0 } };
        end
      else
        begin
          if (s_r_line_read_addr2_reg >= 11'd5 || s_r_line_read_addr3_reg >= 11'd5 || s_r_line_read_addr4_reg >= 11'd5 || s_r_line_read_addr1_reg >= 11'd5 || s_r_line_read_addr0_reg >= 11'd5 || extd_flag)
            begin
              s_r_p22_MedIn22 <= s_r_p22_i;
            end
        end
    end
  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          out_cntr_fsm <= cntr_start;
          output_pix_cntr <= 11'd0;
          data_Vld_out <= 1'b0;
        end
      else
        begin
          case (out_cntr_fsm)
            cntr_start:
              begin
              
                if (s_r_Outpixel_cnt == 11'd8)
                  begin
                    output_pix_cntr <= output_pix_cntr + 1'b1;
                    data_Vld_out <= 1'b1;
                    out_cntr_fsm <= cntr_run;
                  end
                else
                  begin
                    out_cntr_fsm <= cntr_start;
                    output_pix_cntr <= 11'd0;
                    data_Vld_out <= 1'b0;
                  end
              end
            cntr_run:
              begin
                if (output_pix_cntr == g_DISPLAY_RESOLUTION)
                  begin
                    out_cntr_fsm <= cntr_start;
                    output_pix_cntr <= 11'd0;
                    data_Vld_out <= 1'b0;
                  end
                else
                  begin
                    output_pix_cntr <= output_pix_cntr + 1'b1;
                    data_Vld_out <= 1'b1;
                    out_cntr_fsm <= cntr_run;
                  end
              end
            default:
              begin
                out_cntr_fsm <= cntr_start;
              end
          endcase
        end
    end
  


  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          Y_Out_o <= 24'd0;
          data_Vld_out_o <= 1'b0;
        end
      else
        begin
          Y_Out_o <= s_r_p22_MedIn22_d12;
          data_Vld_out_o <= data_Vld_out_d12;
        end
    end


  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          s_r_p22_MedIn22_d <= 24'd0;
          s_r_p22_MedIn22_d1 <= 24'd0;
          s_r_p22_MedIn22_d2 <= 24'd0;
          s_r_p22_MedIn22_d3 <= 24'd0;
          s_r_p22_MedIn22_d4 <= 24'd0;
          s_r_p22_MedIn22_d5 <= 24'd0;
          s_r_p22_MedIn22_d6 <= 24'd0;
          s_r_p22_MedIn22_d7 <= 24'd0;
          s_r_p22_MedIn22_d8 <= 24'd0;
          s_r_p22_MedIn22_d9 <= 24'd0;
          s_r_p22_MedIn22_d10 <= 24'd0;
          s_r_p22_MedIn22_d11 <= 24'd0;
          s_r_p22_MedIn22_d12 <= 24'd0;
        end
      else
        begin
          s_r_p22_MedIn22_d <= s_r_p22_MedIn22;
          s_r_p22_MedIn22_d1 <= s_r_p22_MedIn22_d;
          s_r_p22_MedIn22_d2 <= s_r_p22_MedIn22_d1;
          s_r_p22_MedIn22_d3 <= s_r_p22_MedIn22_d2;
          s_r_p22_MedIn22_d4 <= s_r_p22_MedIn22_d3;
          s_r_p22_MedIn22_d5 <= s_r_p22_MedIn22_d4;
          s_r_p22_MedIn22_d6 <= s_r_p22_MedIn22_d5;
          s_r_p22_MedIn22_d7 <= s_r_p22_MedIn22_d6;
          s_r_p22_MedIn22_d8 <= s_r_p22_MedIn22_d7;
          s_r_p22_MedIn22_d9 <= s_r_p22_MedIn22_d8;
          s_r_p22_MedIn22_d10 <= s_r_p22_MedIn22_d9;
          s_r_p22_MedIn22_d11 <= s_r_p22_MedIn22_d10;
          s_r_p22_MedIn22_d12 <= s_r_p22_MedIn22_d11;
        end
    end


  always
    @(posedge SYS_CLK_I or
      negedge RESET_n_I)
    begin
      if (!RESET_n_I)
        begin
          data_Vld_out_d  <= 1'd0;
          data_Vld_out_d1 <= 1'd0;
          data_Vld_out_d2 <= 1'd0;
          data_Vld_out_d3 <= 1'd0;
          data_Vld_out_d4 <= 1'd0;
          data_Vld_out_d5 <= 1'd0;
          data_Vld_out_d6  <= 1'd0;
          data_Vld_out_d7 <= 1'd0;
          data_Vld_out_d8 <= 1'd0;
          data_Vld_out_d9 <= 1'd0;
          data_Vld_out_d10 <= 1'd0; 
          data_Vld_out_d11 <= 1'd0;
          data_Vld_out_d12 <= 1'd0;    
        end
      else
        begin
          data_Vld_out_d <= data_Vld_out;
          data_Vld_out_d1 <= data_Vld_out_d;
          data_Vld_out_d2 <= data_Vld_out_d1;
          data_Vld_out_d3 <= data_Vld_out_d2;
          data_Vld_out_d4 <= data_Vld_out_d3;
          data_Vld_out_d5 <= data_Vld_out_d4;
          data_Vld_out_d6 <= data_Vld_out_d5;
          data_Vld_out_d7 <= data_Vld_out_d6;
          data_Vld_out_d8 <= data_Vld_out_d7;
          data_Vld_out_d9 <= data_Vld_out_d8;
          data_Vld_out_d10 <= data_Vld_out_d9;
          data_Vld_out_d11 <= data_Vld_out_d10;
          data_Vld_out_d12 <= data_Vld_out_d11;
        end
    end


  //Input Line buffer Zero
  ramDualPort #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH)) Line_Buffer0_i(.clk(SYS_CLK_I),
                                                                                        .addr_a(s_r_line_write_addr),
                                                                                        .data_a(s_r_data_in),
                                                                                        .we_a(s_r_we0),
                                                                                        .addr_b(s_r_line_read_addr0),
                                                                                        .we_b(s_r_re0),
                                                                                        .q_b(s_r_read_data0));
  //Input Line buffer One
  ramDualPort #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH)) Line_Buffer1_i(.clk(SYS_CLK_I),
                                                                                        .addr_a(s_r_line_write_addr),
                                                                                        .data_a(s_r_data_in),
                                                                                        .we_a(s_r_we1),
                                                                                        .addr_b(s_r_line_read_addr1),
                                                                                        .we_b(s_r_re1),
                                                                                        .q_b(s_r_read_data1));
  //Input Line buffer Two
  ramDualPort #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH)) Line_Buffer2_i(.clk(SYS_CLK_I),
                                                                                        .addr_a(s_r_line_write_addr),
                                                                                        .data_a(s_r_data_in),
                                                                                        .we_a(s_r_we2),
                                                                                        .addr_b(s_r_line_read_addr2),
                                                                                        .we_b(s_r_re2),
                                                                                        .q_b(s_r_read_data2));
  //Input Line buffer Three
  ramDualPort #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH)) Line_Buffer3_i(.clk(SYS_CLK_I),
                                                                                        .addr_a(s_r_line_write_addr),
                                                                                        .data_a(s_r_data_in),
                                                                                        .we_a(s_r_we3),
                                                                                        .addr_b(s_r_line_read_addr3),
                                                                                        .we_b(s_r_re3),
                                                                                        .q_b(s_r_read_data3));
  //Input Line buffer Three
  ramDualPort #(.DATA_WIDTH(g_DATAWIDTH), .ADDRESS_WIDTH(g_X_RES_WIDTH)) Line_Buffer4_i(.clk(SYS_CLK_I),
                                                                                        .addr_a(s_r_line_write_addr),
                                                                                        .data_a(s_r_data_in),
                                                                                        .we_a(s_r_we4),
                                                                                        .addr_b(s_r_line_read_addr4),
                                                                                        .we_b(s_r_re4),
                                                                                        .q_b(s_r_read_data4));
				       
endmodule
